"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonlDB = void 0;
const lockfile = __importStar(require("@alcalzone/proper-lockfile"));
const async_1 = require("alcalzone-shared/async");
const deferred_promise_1 = require("alcalzone-shared/deferred-promise");
const objects_1 = require("alcalzone-shared/objects");
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const readline = __importStar(require("readline"));
var Operation;
(function (Operation) {
    Operation[Operation["Clear"] = 0] = "Clear";
    Operation[Operation["Write"] = 1] = "Write";
    Operation[Operation["Delete"] = 2] = "Delete";
})(Operation || (Operation = {}));
/**
 * fsync on a directory ensures there are no rename operations etc. which haven't been persisted to disk.
 */
/* istanbul ignore next - This is impossible to test */
async function fsyncDir(dirname) {
    // Windows will cause `EPERM: operation not permitted, fsync`
    // for directories, so don't do this
    if (process.platform === "win32")
        return;
    const fd = await fs.open(dirname, "r");
    await fs.fsync(fd);
    await fs.close(fd);
}
function getCurrentErrorStack() {
    const tmp = { message: "" };
    Error.captureStackTrace(tmp);
    return tmp.stack.split("\n").slice(2).join("\n");
}
class JsonlDB {
    constructor(filename, options = {}) {
        var _a, _b;
        this._db = new Map();
        this._uncompressedSize = Number.NaN;
        this._changesSinceLastCompress = 0;
        this._isOpen = false;
        this._persistenceTasks = [];
        this._journal = [];
        this.validateOptions(options);
        this.filename = filename;
        this.dumpFilename = this.filename + ".dump";
        this.backupFilename = this.filename + ".bak";
        const lockfileDirectory = (_b = (_a = options.lockfile) === null || _a === void 0 ? void 0 : _a.directory) !== null && _b !== void 0 ? _b : options.lockfileDirectory;
        this.lockfileName = lockfileDirectory
            ? path.join(lockfileDirectory, path.basename(this.filename))
            : this.filename;
        this.options = options;
        // Bind all map properties we can use directly
        this.forEach = this._db.forEach.bind(this._db);
        this.get = this._db.get.bind(this._db);
        this.has = this._db.has.bind(this._db);
        this.entries = this._db.entries.bind(this._db);
        this.keys = this._db.keys.bind(this._db);
        this.values = this._db.values.bind(this._db);
        this[Symbol.iterator] = this._db[Symbol.iterator].bind(this._db);
    }
    validateOptions(options) {
        if (options.autoCompress) {
            const { sizeFactor, sizeFactorMinimumSize, intervalMs, intervalMinChanges, } = options.autoCompress;
            if (sizeFactor != undefined && sizeFactor <= 1) {
                throw new Error("sizeFactor must be > 1");
            }
            if (sizeFactorMinimumSize != undefined &&
                sizeFactorMinimumSize < 0) {
                throw new Error("sizeFactorMinimumSize must be >= 0");
            }
            if (intervalMs != undefined && intervalMs < 10) {
                throw new Error("intervalMs must be >= 10");
            }
            if (intervalMinChanges != undefined && intervalMinChanges < 1) {
                throw new Error("intervalMinChanges must be >= 1");
            }
        }
        if (options.throttleFS) {
            const { intervalMs, maxBufferedCommands } = options.throttleFS;
            if (intervalMs < 0) {
                throw new Error("intervalMs must be >= 0");
            }
            if (maxBufferedCommands != undefined && maxBufferedCommands < 0) {
                throw new Error("maxBufferedCommands must be >= 0");
            }
        }
        if (options.lockfile) {
            const { directory, retries, staleMs = 10000, updateMs = staleMs / 2, retryMinTimeoutMs, } = options.lockfile;
            if (staleMs < 2000) {
                throw new Error("staleMs must be >= 2000");
            }
            if (updateMs < 1000) {
                throw new Error("updateMs must be >= 1000");
            }
            if (updateMs > staleMs / 2) {
                throw new Error(`updateMs must be <= ${staleMs / 2}`);
            }
            if (retries != undefined && retries < 0) {
                throw new Error("retries must be >= 0");
            }
            if (retries != undefined && retries > 10) {
                throw new Error("retries must be <= 10");
            }
            if (retryMinTimeoutMs != undefined && retryMinTimeoutMs < 100) {
                throw new Error("retryMinTimeoutMs must be >= 100");
            }
            if (options.lockfileDirectory != undefined &&
                directory != undefined) {
                throw new Error("lockfileDirectory and lockfile.directory must not both be specified");
            }
        }
    }
    get size() {
        return this._db.size;
    }
    /** Returns the line count of the appendonly file, excluding empty lines */
    get uncompressedSize() {
        if (!this._isOpen) {
            throw new Error("The database is not open!");
        }
        return this._uncompressedSize;
    }
    get isOpen() {
        return this._isOpen;
    }
    drainJournal() {
        return this._journal.splice(0, this._journal.length);
    }
    // /** Opens the database file or creates it if it doesn't exist */
    async open() {
        var _a, _b, _c, _d, _e, _f;
        // Open the file for appending and reading
        await fs.ensureDir(path.dirname(this.filename));
        let retryOptions;
        if ((_a = this.options.lockfile) === null || _a === void 0 ? void 0 : _a.retries) {
            retryOptions = {
                minTimeout: (_b = this.options.lockfile.retryMinTimeoutMs) !== null && _b !== void 0 ? _b : ((_c = this.options.lockfile.updateMs) !== null && _c !== void 0 ? _c : 2000) / 2,
                retries: this.options.lockfile.retries,
                factor: 1.25,
            };
        }
        try {
            await fs.ensureDir(path.dirname(this.lockfileName));
            await lockfile.lock(this.lockfileName, {
                // We cannot be sure that the file exists before acquiring the lock
                realpath: false,
                stale: 
                // Avoid timeouts during testing
                process.env.NODE_ENV === "test"
                    ? 100000
                    : /* istanbul ignore next - this is impossible to test */ (_d = this
                        .options.lockfile) === null || _d === void 0 ? void 0 : _d.staleMs,
                update: (_e = this.options.lockfile) === null || _e === void 0 ? void 0 : _e.updateMs,
                retries: retryOptions,
                onCompromised: /* istanbul ignore next */ () => {
                    // do nothing
                },
            });
        }
        catch (e) {
            throw new Error(`Failed to lock DB file "${this.lockfileName}"!`);
        }
        // If the application crashed previously, try to recover from it
        await this.tryRecoverDBFiles();
        this._fd = await fs.open(this.filename, "a+");
        const readStream = fs.createReadStream(this.filename, {
            encoding: "utf8",
            fd: this._fd,
            autoClose: false,
        });
        const rl = readline.createInterface(readStream);
        let lineNo = 0;
        this._uncompressedSize = 0;
        try {
            await new Promise((resolve, reject) => {
                const actualLines = new Map();
                rl.on("line", (line) => {
                    // Count source lines for the error message
                    lineNo++;
                    // Skip empty lines
                    if (!line)
                        return;
                    try {
                        this._uncompressedSize++;
                        // Extract the key and only remember the last line for each one
                        const key = this.parseKey(line);
                        actualLines.set(key, [lineNo, line]);
                    }
                    catch (e) {
                        if (this.options.ignoreReadErrors === true) {
                            return;
                        }
                        else {
                            reject(new Error(`Cannot open file: Invalid data in line ${lineNo}`));
                        }
                    }
                });
                rl.on("close", () => {
                    // We've read all lines, now only parse those that contain useful data
                    for (const [lineNo, line] of actualLines.values()) {
                        try {
                            this.parseLine(line);
                        }
                        catch (e) {
                            if (this.options.ignoreReadErrors === true) {
                                continue;
                            }
                            else {
                                reject(new Error(`Cannot open file: Invalid data in line ${lineNo}`));
                            }
                        }
                    }
                    resolve();
                });
            });
        }
        finally {
            // Close the file again to avoid EBADF
            rl.close();
            await fs.close(this._fd);
            this._fd = undefined;
        }
        // Start background persistence thread
        this._persistencePromise = this.persistenceThread();
        await this._openPromise;
        this._isOpen = true;
        // If the DB should be compressed while opening, do it now
        if ((_f = this.options.autoCompress) === null || _f === void 0 ? void 0 : _f.onOpen)
            await this.compress();
    }
    /**
     * Makes sure that there are no remains of a previous broken compress attempt and restores
     * a DB backup if it exists.
     */
    async tryRecoverDBFiles() {
        // During the compression, the following sequence of events happens:
        // 1. A .jsonl.dump file gets written with a compressed copy of the data
        // 2. Files get renamed: .jsonl -> .jsonl.bak, .jsonl.dump -> .jsonl
        // 3. .bak file gets removed
        // 4. Buffered data gets written to the .jsonl file
        // This means if the .jsonl file is absent or truncated, we should be able to pick either the .dump or the .bak file
        // and restore the .jsonl file from it
        let dbFileIsOK = false;
        try {
            const dbFileStats = await fs.stat(this.filename);
            dbFileIsOK = dbFileStats.isFile() && dbFileStats.size > 0;
        }
        catch {
            // ignore
        }
        // Prefer the DB file if it exists, remove the others in case they exist
        if (dbFileIsOK) {
            try {
                await fs.remove(this.backupFilename);
            }
            catch {
                // ignore
            }
            try {
                await fs.remove(this.dumpFilename);
            }
            catch {
                // ignore
            }
            return;
        }
        // The backup file should have complete data - the dump file could be subject to an incomplete write
        let bakFileIsOK = false;
        try {
            const bakFileStats = await fs.stat(this.backupFilename);
            bakFileIsOK = bakFileStats.isFile() && bakFileStats.size > 0;
        }
        catch {
            // ignore
        }
        if (bakFileIsOK) {
            // Overwrite the broken db file with it and delete the dump file
            try {
                await fs.move(this.backupFilename, this.filename, {
                    overwrite: true,
                });
                try {
                    await fs.remove(this.dumpFilename);
                }
                catch {
                    // ignore
                }
                return;
            }
            catch {
                // Moving failed, try the next possibility
            }
        }
        // Try the dump file as a last attempt
        let dumpFileIsOK = false;
        try {
            const dumpFileStats = await fs.stat(this.dumpFilename);
            dumpFileIsOK = dumpFileStats.isFile() && dumpFileStats.size > 0;
        }
        catch {
            // ignore
        }
        if (dumpFileIsOK) {
            try {
                // Overwrite the broken db file with the dump file and delete the backup file
                await fs.move(this.dumpFilename, this.filename, {
                    overwrite: true,
                });
                try {
                    await fs.remove(this.backupFilename);
                }
                catch {
                    // ignore
                }
                return;
            }
            catch {
                // Moving failed
            }
        }
    }
    /** Reads a line and extracts the key without doing a full-blown JSON.parse() */
    parseKey(line) {
        if (0 !== line.indexOf(`{"k":"`)) {
            throw new Error("invalid data");
        }
        const keyStart = 6;
        let keyEnd = line.indexOf(`","v":`, keyStart);
        if (-1 === keyEnd) {
            // try again with a delete command
            if (line.endsWith(`"}`)) {
                keyEnd = line.length - 2;
            }
            else {
                throw new Error("invalid data");
            }
        }
        return line.slice(keyStart, keyEnd);
    }
    /** Parses a line and updates the internal DB correspondingly */
    parseLine(line) {
        const record = JSON.parse(line);
        const { k, v } = record;
        if (v !== undefined) {
            this._db.set(k, typeof this.options.reviver === "function"
                ? this.options.reviver(k, v)
                : v);
        }
        else {
            this._db.delete(k);
        }
    }
    clear() {
        if (!this._isOpen) {
            throw new Error("The database is not open!");
        }
        this._db.clear();
        // All pending writes are obsolete, remove them from the journal
        this.drainJournal();
        this._journal.push(this.makeLazyClear());
    }
    delete(key) {
        if (!this._isOpen) {
            throw new Error("The database is not open!");
        }
        const ret = this._db.delete(key);
        if (ret) {
            // Something was deleted
            this._journal.push(this.makeLazyDelete(key));
        }
        return ret;
    }
    set(key, value) {
        if (!this._isOpen) {
            throw new Error("The database is not open!");
        }
        this._db.set(key, value);
        this._journal.push(this.makeLazyWrite(key, value));
        return this;
    }
    async importJsonFile(filename) {
        const json = await fs.readJSON(filename);
        return this.importJson(json);
    }
    importJson(jsonOrFile) {
        if (typeof jsonOrFile === "string") {
            if (!this._isOpen) {
                return Promise.reject(new Error("The database is not open!"));
            }
            return this.importJsonFile(jsonOrFile);
        }
        else {
            if (!this._isOpen) {
                throw new Error("The database is not open!");
            }
        }
        for (const [key, value] of Object.entries(jsonOrFile)) {
            this._db.set(key, value);
            this._journal.push(this.makeLazyWrite(key, value));
        }
    }
    async exportJson(filename, options) {
        if (!this._isOpen) {
            return Promise.reject(new Error("The database is not open!"));
        }
        return fs.writeJSON(filename, (0, objects_1.composeObject)([...this._db]), options);
    }
    entryToLine(key, value) {
        var _a, _b, _c;
        if (value !== undefined) {
            return JSON.stringify({
                k: key,
                v: (_c = (_b = (_a = this.options).serializer) === null || _b === void 0 ? void 0 : _b.call(_a, key, value)) !== null && _c !== void 0 ? _c : value,
            });
        }
        else {
            return JSON.stringify({ k: key });
        }
    }
    makeLazyClear() {
        return {
            op: Operation.Clear,
            serialize: 
            /* istanbul ignore next - this is impossible to test since it requires exact timing */ () => "",
        };
    }
    makeLazyDelete(key) {
        let serialized;
        return {
            op: Operation.Delete,
            key,
            serialize: () => {
                if (serialized == undefined) {
                    serialized = this.entryToLine(key);
                }
                return serialized;
            },
        };
    }
    makeLazyWrite(key, value) {
        let serialized;
        return {
            op: Operation.Write,
            key,
            value,
            serialize: () => {
                if (serialized == undefined) {
                    serialized = this.entryToLine(key, value);
                }
                return serialized;
            },
        };
    }
    /**
     * Saves a compressed copy of the DB into the given path.
     *
     * **WARNING:** This MUST be called from {@link persistenceThread}!
     * @param targetFilename Where the compressed copy should be written. Default: `<filename>.dump`
     * @param drainJournal Whether the journal should be drained when writing the compressed copy or simply cloned.
     */
    async dumpInternal(targetFilename = this.dumpFilename, drainJournal) {
        // Open the file for writing (or truncate if it exists)
        const fd = await fs.open(targetFilename, "w+");
        // Create a copy of the other entries in the DB
        // Also, remember how many entries were in the journal. These are already part of
        // the map, so we don't need to append them later and keep a consistent state
        const entries = [...this._db];
        const journalLength = this._journal.length;
        // And persist them
        let serialized = "";
        for (const [key, value] of entries) {
            // No need to serialize lazily here
            serialized += this.entryToLine(key, value) + "\n";
        }
        await fs.appendFile(fd, serialized);
        // In case there is any new data in the journal, persist that too
        let journal = drainJournal
            ? this._journal.splice(0, this._journal.length)
            : this._journal;
        journal = journal.slice(journalLength);
        await this.writeJournalToFile(fd, journal, false);
        await fs.close(fd);
    }
    /**
     * Saves a compressed copy of the DB into the given path.
     * @param targetFilename Where the compressed copy should be written. Default: `<filename>.dump`
     */
    async dump(targetFilename = this.dumpFilename) {
        // Prevent dumping the DB when it is closed
        if (!this._isOpen)
            return;
        const done = (0, deferred_promise_1.createDeferredPromise)();
        this._persistenceTasks.push({
            type: "dump",
            filename: targetFilename,
            done,
        });
        const stack = getCurrentErrorStack();
        try {
            await done;
        }
        catch (e) {
            e.stack += "\n" + stack;
            throw e;
        }
    }
    needToCompressBySize() {
        var _a;
        const { sizeFactor = Number.POSITIVE_INFINITY, sizeFactorMinimumSize = 0, } = (_a = this.options.autoCompress) !== null && _a !== void 0 ? _a : {};
        if (this._uncompressedSize >= sizeFactorMinimumSize &&
            this._uncompressedSize >= sizeFactor * this.size) {
            return true;
        }
        return false;
    }
    needToCompressByTime(lastCompress) {
        if (!this.options.autoCompress)
            return false;
        const { intervalMs, intervalMinChanges = 1 } = this.options.autoCompress;
        if (!intervalMs)
            return false;
        return (this._changesSinceLastCompress >= intervalMinChanges &&
            Date.now() - lastCompress >= intervalMs);
    }
    async persistenceThread() {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        // Keep track of the write accesses and compression attempts
        let lastWrite = Date.now();
        let lastCompress = Date.now();
        const throttleInterval = (_b = (_a = this.options.throttleFS) === null || _a === void 0 ? void 0 : _a.intervalMs) !== null && _b !== void 0 ? _b : 0;
        const maxBufferedCommands = (_d = (_c = this.options.throttleFS) === null || _c === void 0 ? void 0 : _c.maxBufferedCommands) !== null && _d !== void 0 ? _d : Number.POSITIVE_INFINITY;
        // Open the file for appending and reading
        this._fd = await fs.open(this.filename, "a+");
        (_e = this._openPromise) === null || _e === void 0 ? void 0 : _e.resolve();
        const sleepDuration = 20; // ms
        while (true) {
            // Figure out what to do
            let task;
            if (this.needToCompressBySize() ||
                this.needToCompressByTime(lastCompress)) {
                // Need to compress
                task = { type: "compress", done: (0, deferred_promise_1.createDeferredPromise)() };
                // but catch errors!
                // eslint-disable-next-line @typescript-eslint/no-empty-function
                task.done.catch(() => { });
            }
            else {
                // Take the first tasks of from the task queue
                task = (_f = this._persistenceTasks.shift()) !== null && _f !== void 0 ? _f : { type: "none" };
            }
            let isStopCmd = false;
            switch (task.type) {
                case "stop":
                    isStopCmd = true;
                // fall through
                case "none": {
                    // Write to disk if necessary
                    const shouldWrite = this._journal.length > 0 &&
                        (isStopCmd ||
                            Date.now() - lastWrite > throttleInterval ||
                            this._journal.length > maxBufferedCommands);
                    if (shouldWrite) {
                        // Drain the journal
                        const journal = this.drainJournal();
                        this._fd = await this.writeJournalToFile(this._fd, journal);
                        lastWrite = Date.now();
                    }
                    if (isStopCmd) {
                        await fs.close(this._fd);
                        this._fd = undefined;
                        return;
                    }
                    break;
                }
                case "dump": {
                    try {
                        await this.dumpInternal(task.filename, false);
                        task.done.resolve();
                    }
                    catch (e) {
                        task.done.reject(e);
                    }
                    break;
                }
                case "compress": {
                    try {
                        await this.doCompress();
                        lastCompress = Date.now();
                        (_g = task.done) === null || _g === void 0 ? void 0 : _g.resolve();
                    }
                    catch (e) {
                        (_h = task.done) === null || _h === void 0 ? void 0 : _h.reject(e);
                    }
                    break;
                }
            }
            await (0, async_1.wait)(sleepDuration);
        }
    }
    /** Writes the given journal to the given file descriptor. Returns the new file descriptor if the file was re-opened during the process */
    async writeJournalToFile(fd, journal, updateStatistics = true) {
        // The chunk map is used to buffer all entries that are currently waiting in line
        // so we avoid serializing redundant entries. When the writing is throttled,
        // the chunk map will only be used for a short time.
        const chunk = new Map();
        let serialized = "";
        let truncate = false;
        for (const entry of journal) {
            if (entry.op === Operation.Clear) {
                chunk.clear();
                truncate = true;
            }
            else {
                // Only remember the last entry for each key
                chunk.set(entry.key, entry);
            }
        }
        // When the journal has been drained, perform the necessary write actions
        if (truncate) {
            // Since we opened the file in append mode, we cannot truncate
            // therefore close and open in write mode again
            await fs.close(fd);
            fd = await fs.open(this.filename, "w+");
            truncate = false;
            if (updateStatistics) {
                // Now the DB size is effectively 0 and we have no "uncompressed" changes pending
                this._uncompressedSize = 0;
                this._changesSinceLastCompress = 0;
            }
        }
        // Collect all changes
        for (const entry of chunk.values()) {
            serialized += entry.serialize() + "\n";
            if (updateStatistics) {
                this._uncompressedSize++;
                this._changesSinceLastCompress++;
            }
        }
        // and write once, making sure everything is written
        await fs.appendFile(fd, serialized);
        await fs.fsync(fd);
        return fd;
    }
    /**
     * Compresses the db by dumping it and overwriting the aof file.
     *
     * **WARNING:** This MUST be called from {@link persistenceThread}!
     */
    async doCompress() {
        // 1. Ensure the backup contains everything in the DB and journal
        const journal = this.drainJournal();
        this._fd = await this.writeJournalToFile(this._fd, journal);
        await fs.close(this._fd);
        this._fd = undefined;
        // 2. Create a dump, draining the journal to avoid duplicate writes
        await this.dumpInternal(this.dumpFilename, true);
        // 3. Ensure there are no pending rename operations or file creations
        await fsyncDir(path.dirname(this.filename));
        // 4. Swap files around, then ensure the directory entries are written to disk
        await fs.move(this.filename, this.backupFilename, {
            overwrite: true,
        });
        await fs.move(this.dumpFilename, this.filename, { overwrite: true });
        await fsyncDir(path.dirname(this.filename));
        // 5. Delete backup
        await fs.unlink(this.backupFilename);
        // 6. open the main DB file again in append mode
        this._fd = await fs.open(this.filename, "a+");
        // Remember the new statistics
        this._uncompressedSize = this._db.size;
        this._changesSinceLastCompress = 0;
    }
    /** Compresses the db by dumping it and overwriting the aof file. */
    async compress() {
        if (!this._isOpen)
            return;
        await this.compressInternal();
    }
    /** Compresses the db by dumping it and overwriting the aof file. */
    async compressInternal() {
        // Avoid having multiple compress operations running in parallel
        const task = this._persistenceTasks.find((t) => t.type === "compress");
        if (task)
            return task.done;
        const done = (0, deferred_promise_1.createDeferredPromise)();
        this._persistenceTasks.push({
            type: "compress",
            done,
        });
        const stack = getCurrentErrorStack();
        try {
            await done;
        }
        catch (e) {
            e.stack += "\n" + stack;
            throw e;
        }
    }
    /** Closes the DB and waits for all data to be written */
    async close() {
        var _a;
        if (!this._isOpen)
            return;
        this._isOpen = false;
        // Compress on close if required
        if ((_a = this.options.autoCompress) === null || _a === void 0 ? void 0 : _a.onClose) {
            await this.compressInternal();
        }
        // Stop persistence thread and wait for it to finish
        this._persistenceTasks.push({ type: "stop" });
        await this._persistencePromise;
        // Reset all variables
        this._db.clear();
        this._changesSinceLastCompress = 0;
        this._uncompressedSize = Number.NaN;
        // Free the lock
        try {
            if (await lockfile.check(this.lockfileName, { realpath: false }))
                await lockfile.unlock(this.lockfileName, { realpath: false });
        }
        catch {
            // whatever - just don't crash
        }
    }
}
exports.JsonlDB = JsonlDB;
//# sourceMappingURL=db.js.map