"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertLogInfo = exports.assertMessage = exports.SpyTransport = void 0;
const ansi_colors_1 = require("ansi-colors");
const triple_beam_1 = require("triple-beam");
const winston_transport_1 = __importDefault(require("winston-transport"));
const timestampRegex = /\d{2}\:\d{2}\:\d{2}\.\d{3}/g;
const timestampPrefixRegex = new RegExp(`^(${ansi_colors_1.ansiRegex.source})?${timestampRegex.source}(${ansi_colors_1.ansiRegex.source})? `, "gm");
const channelRegex = /(SERIAL|CNTRLR|DRIVER|RFLCTN)/g;
const channelPrefixRegex = new RegExp(`(${ansi_colors_1.ansiRegex.source})?${channelRegex.source}(${ansi_colors_1.ansiRegex.source})? `, "gm");
/** Log to a jest.fn() in order to perform assertions during unit tests */
class SpyTransport extends winston_transport_1.default {
    constructor() {
        super({
            level: "silly",
        });
        this._spy = jest.fn();
    }
    get spy() {
        return this._spy;
    }
    log(info, next) {
        this._spy(info);
        next();
    }
}
exports.SpyTransport = SpyTransport;
/** Tests a printed log message */
function assertMessage(transport, options) {
    const callNumber = options.callNumber || 0;
    expect(transport.spy.mock.calls.length).toBeGreaterThan(callNumber);
    const callArg = transport.spy.mock.calls[callNumber][0];
    let actualMessage = callArg[triple_beam_1.MESSAGE];
    // By default ignore the color codes
    const ignoreColor = options.ignoreColor !== false;
    if (ignoreColor) {
        actualMessage = (0, ansi_colors_1.stripColor)(actualMessage);
    }
    // By default, strip away the timestamp and placeholder
    if (options.ignoreTimestamp !== false) {
        actualMessage = actualMessage
            .replace(timestampPrefixRegex, "")
            .replace(/^ {13}/gm, "");
    }
    // by default, strip away the channel label and placeholder
    if (options.ignoreChannel !== false) {
        actualMessage = actualMessage
            .replace(channelPrefixRegex, "")
            .replace(/^ {7}/gm, "");
    }
    if (typeof options.message === "string") {
        if (ignoreColor) {
            options.message = (0, ansi_colors_1.stripColor)(options.message);
        }
        expect(actualMessage).toEqual(options.message);
    }
    if (typeof options.predicate === "function") {
        expect(actualMessage).toSatisfy(options.predicate);
    }
}
exports.assertMessage = assertMessage;
function assertLogInfo(transport, options) {
    const callNumber = options.callNumber || 0;
    expect(transport.spy.mock.calls.length).toBeGreaterThan(callNumber);
    const callArg = transport.spy.mock.calls[callNumber][0];
    if (typeof options.level === "string") {
        expect(callArg.level).toEqual(options.level);
    }
    if (typeof options.predicate === "function") {
        expect(callArg).toSatisfy(options.predicate);
    }
}
exports.assertLogInfo = assertLogInfo;
//# sourceMappingURL=SpyTransport.js.map